//
//  NavigationViewController.m
//  ShipwreckAdventure
//
//  Created by Cody Powell on 11/9/10.
//  Copyright 2010 Famigo. All rights reserved.
//

#import "NavigationViewController.h"
#import "Quest.h"
#import "Famigo.h"
#import "Player.h"

#define LOCATION_COUNT 5

@interface NavigationViewController()
	-(NSString*)getDescriptionForLocation:(LocationDesc)location;
@end

@implementation NavigationViewController

@synthesize newLocation;
/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
	UIColor *blue = [UIColor colorWithRed:(0./255.) green:(180./255.) blue:(230./255.) alpha:1.];;
	[moveButton setStartColor:blue];
	[moveButton setEndColor:blue];
	[moveButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
	[moveButton setTitleColor:[UIColor whiteColor] forState:UIControlStateHighlighted];
	[self refreshView];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

-(void)refreshView {
	Quest *q = [Quest sharedInstance];
	switch (q.currentStage) {
		case Organize:
			taskLabel.text = @"Task: Find Crew";
			recommendLabel.text = @"Recommend: Use Chat";
			
			Famigo *f = [Famigo sharedInstance];
			Quest *q = [Quest sharedInstance];
			
			// Where am I?
			NSString *currentMemberId = f.member_id;
			LocationDesc myLocation = Nowhere;
			NSMutableArray *playersAtSameLocation = [[NSMutableArray alloc] init];
			NSString *playerNamesAtSameLocation = nil;
			
			for (Player *player in q.players) {
				if ([player.memberId isEqualToString:currentMemberId]) {
					myLocation = player.currentLocation;
				}
			}
			
			// Is there anyone there with me?
			for (Player *player in q.players) {
				if (player.currentLocation == myLocation && (![player.memberId isEqualToString:currentMemberId])) {
					[playersAtSameLocation addObject:player.memberId];
				}
			}
		
			for (NSString *playerAtSameLocation in playersAtSameLocation) {
				for (NSDictionary *playerDict in [f.gameInstance valueForKey:FC_d_game_players]) {
					NSString *memberId = [playerDict valueForKey:FC_d_member_id];
					NSString *memberName = [playerDict valueForKey:FC_d_member_name];
					
					if ([memberId isEqualToString:playerAtSameLocation]) {
						if (!playerNamesAtSameLocation)
							playerNamesAtSameLocation = [NSString stringWithFormat:@"%@",memberName];
						else
							playerNamesAtSameLocation = [NSString stringWithFormat:@"%@, %@", playerNamesAtSameLocation, memberName];
					}
				}
			}
			
			NSString *currentStatus = [NSString stringWithFormat:@"You're at the %@.",[self getDescriptionForLocation:myLocation]];
			if (playerNamesAtSameLocation)
				currentStatus = [NSString stringWithFormat:@"%@  You're joined there by %@.", currentStatus, playerNamesAtSameLocation];
			else
				currentStatus = [NSString stringWithFormat:@"%@  You're by yourself.", currentStatus];

			currentPlayerStatus.text = currentStatus;
			break;
		default:
			NSAssert(0, @"You shouldn't be here duing this stage!");
	}
}

- (IBAction)moveButtonClicked:(id)sender {
	if (self.newLocation == Nowhere)
		return;
	
	// First, find my player and move him to my new location.
	Famigo *f = [Famigo sharedInstance];
	NSString *currentMemberId = f.member_id;
	Quest *q = [Quest sharedInstance];
	
	for (Player *player in q.players) {
		if ([player.memberId isEqualToString:currentMemberId]) {
			player.currentLocation = self.newLocation;
		}
	}
	
	// If all players are at the same location, we move to the next level.  Woohoo!
	if ([q allPlayersAtSameLocation]) {
		q.currentStage++;
	}
	
	// Update the game regardless.
	NSDictionary *gameDictionary = [q buildDictionaryForCurrentQuest];
	[f.gameInstance setValue:gameDictionary forKey:f.game_name];
	[f updateGame];
}

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)thePickerView {
	return 1;
}

- (NSInteger)pickerView:(UIPickerView *)thePickerView numberOfRowsInComponent:(NSInteger)component {
	return LOCATION_COUNT;
}

- (NSString *)pickerView:(UIPickerView *)thePickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
	// Nowhere isn't a valid location, so we start from 1 here.
	LocationDesc location = row + 1;
	return [self getDescriptionForLocation:location];
}

-(NSString*)getDescriptionForLocation:(LocationDesc)location {
	switch (location) {
		case MonkeyVillage:
			return @"Monkey Village";
		case Waterfall:
			return @"Waterfall";
		case Forest:
			return @"Forest";
		case BoatLaunch:
			return @"Boat Launch";
		case CrashSite:
			return @"Crash Site";
		default:
			NSAssert(0, @"Picker doesn't support this location!");
			break;
	}
	return @"";
}

- (void)pickerView:(UIPickerView *)thePickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
	self.newLocation = row + 1;
}

@end
