//
//  Quest.m
//  ShipwreckAdventure
//
//  Created by Cody Powell on 11/9/10.
//  Copyright 2010 Famigo. All rights reserved.
//

#import "Quest.h"
#import "Player.h"
#import "Famigo.h"
#import "GameConstants.h"

// Location count does not include nowhere.
#define LOCATION_COUNT 6

@interface Quest()
-(void)buildQuest;
@end

@implementation Quest

static Quest *sharedQuestManager = nil;
@synthesize currentStage, players;

- (id)init {
	if (self = [super init]) {
		[self buildQuest];
	}
	return self;
}

+ (Quest*)sharedInstance {
	@synchronized(self) {
		if (sharedQuestManager == nil) {
			[[self alloc] init]; // assignment not done here
		}
	}
	
	return sharedQuestManager;
}

+ (id)allocWithZone:(NSZone *)zone {
	@synchronized(self) {
		if (sharedQuestManager == nil) {
			sharedQuestManager = [super allocWithZone:zone];
			return sharedQuestManager; // assignment and return on first allocation
		}
	}
	
	return nil; // on subsequent allocation attempts return nil
}

- (id)copyWithZone:(NSZone*)zone {
	return self;
}

- (id)retain {
	return self;
}

- (unsigned)retainCount {
	return UINT_MAX; //denotes an object that cannot be released
}

- (void)release {
	// do nothing
}

- (id)autorelease {
	return self;
}

-(BOOL)allPlayersAtSameLocation {
	LocationDesc previousLocation = 0;
	for(Player *player in self.players) {
		if (previousLocation != Nowhere) {
			if (player.currentLocation != previousLocation) {
				return NO;
			}
		}
		else {
			previousLocation = player.currentLocation;
		}
	}
	
	return YES;
}

-(void)buildQuest {
	self.currentStage = Start;
	
	NSArray *currentPlayers = [[Famigo sharedInstance].gameInstance valueForKey:FC_d_game_players];
	self.players = [[NSMutableArray alloc] init];
	for (NSDictionary *playerDict in currentPlayers) {
		Player *questPlayer = [[Player alloc] init];
		questPlayer.currentItem = Nothing;

		// We never want anyone to start on nowhere.
		questPlayer.currentLocation = (arc4random() % LOCATION_COUNT) + 1;
		questPlayer.memberId = [playerDict valueForKey:FC_d_member_id];
		[self.players addObject:questPlayer];
	}
}

-(NSDictionary*)buildDictionaryForCurrentQuest {		
	NSMutableArray *questPlayers = [[NSMutableArray alloc] init];
	for (Player *player in self.players) {
		NSString *playerMemberId = player.memberId;
		NSDictionary *questPlayer = [NSDictionary dictionaryWithObjectsAndKeys:playerMemberId,FC_d_member_id,
									 [NSNumber numberWithInt:player.currentLocation],FC_player_location,
									 [NSNumber numberWithInt:player.currentItem],FC_player_item,nil];
		[questPlayers addObject:questPlayer];
	}
	
	NSDictionary *gameDictionary = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:self.currentStage],FC_game_stage,
									questPlayers, FC_quest_players, nil];
	NSLog(@"our dictionary is %@", gameDictionary);
	return gameDictionary;
	return nil;
}

-(void)rebuildQuest {
	Famigo *f = [Famigo sharedInstance];
	NSDictionary *gameDictionary = [f.gameInstance valueForKey:f.game_name];
	[self.players removeAllObjects];
	
	for (NSDictionary *questPlayer in [gameDictionary valueForKey:FC_quest_players]) {
		Player *player = [[Player alloc] init];
		player.memberId = [questPlayer valueForKey:FC_d_member_id];
		player.currentItem = [[questPlayer valueForKey:FC_player_item] intValue];
		player.currentLocation = [[questPlayer valueForKey:FC_player_location] intValue];
		[self.players addObject:player];
	}
	self.currentStage = [[gameDictionary valueForKey:FC_game_stage] intValue];
}


@end
