//
//  ShipwreckAdventureViewController.m
//  ShipwreckAdventure
//
//  Created by Cody Powell on 11/9/10.
//  Copyright 2010 Famigo. All rights reserved.
//

#import "ShipwreckAdventureViewController.h"
#import "Famigo.h"
#import "FamigoController.h"
#import "LogoAnimationController.h"
#import "GearTab.h"
#import "Quest.h"
#import "NavigationViewController.h"

@interface ShipwreckAdventureViewController()
- (void) launchFamigoController;
- (void) launchNavigationController;
- (void) showGearTab;
- (void) hideGearTab;
- (void) shipwreckLogoDone;
- (void) buildUIForQuest;
- (void) incrementStageOfQuest;
- (void) launchShipwreckedLogo;
@end

#define G_SUBVIEW_FRAME CGRectMake(0, 0, 320, 460)
#define BackToFamigoButtonIndex 0
#define CancelButtonIndex 1
#define HelpButtonIndex 2
#define GearActionTag 666
#define StartStageTag 667

@implementation ShipwreckAdventureViewController

@synthesize logoController, famigoController, logoView, gearTab, navigationController;
@synthesize previousStage;

// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
    }
    return self;
}

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

- (void)viewDidLoad {
    [super viewDidLoad];
	
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(reachabilityNote:)
												 name:@"kNetworkReachabilityChangedNotification" 
											   object:nil];
	[[Famigo sharedInstance] registerForNotifications:self withSelector:@selector(famigoNote:)];
	self.previousStage = NoStage;
	
	self.logoController = [[LogoAnimationController alloc] init];
	[logoController registerForNotifications:self withSelector:@selector(famigoLogoDone:)];
	
	int gearTabCenterX = 310;
	int gearTabCenterY = 440;
	GearTab *gt = [[GearTab alloc] initWithFrame:CGRectZero];
	gt.center = CGPointMake(gearTabCenterX, gearTabCenterY);
	[gt setAutoresizingMask:(UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleTopMargin)];
	[gt addTarget:self action:@selector(gearAction:) forControlEvents:UIControlEventTouchUpInside];
	[self.view addSubview:gt];
	[gt release];
	self.gearTab = gt;
	gt.enabled = NO;
	gt.alpha = 0;
	
	[self.view addSubview:[logoController view]];
}

- (void)launchShipwreckedLogo {
	CGRect animationRect = CGRectMake(0, 0, self.view.bounds.size.width, self.view.bounds.size.height);
	self.logoView = [[UIImageView alloc] initWithFrame:animationRect];
	[logoView setImage:[UIImage imageNamed:@"shipsplash.png"]];
	[logoView setUserInteractionEnabled:NO];
	[self.view addSubview:logoView];
}

- (void)famigoLogoDone:(NSNotification*)note {
	[self launchShipwreckedLogo];
	
	// Do we want a shipwreck sound?
	/*
	// Need a little bit of buffer between when we hear the Famigo sound and when we hear the Rad Ribs sound.
	double pauseForRrSound = 0.5;
	[self performSelector:@selector(playRrSound) withObject:nil afterDelay:pauseForRrSound]; */
	
	int logoDuration = 2;
	[self performSelector:@selector(shipwreckLogoDone) withObject:nil afterDelay:logoDuration];
}

- (void)shipwreckLogoDone {
	[self.logoView removeFromSuperview];
	[logoView release];
	logoView = nil;
	
	[self launchFamigoController];
}

- (void) launchFamigoController {
	if (self.navigationController)
		[self.navigationController.view removeFromSuperview];
	
	/*
	if (self.waitingStoryController)
		[self.waitingStoryController.view removeFromSuperview]; */
	
	[Famigo sharedInstance].watchGame = NO;
	
	if (!self.famigoController) {
		self.famigoController = [FamigoController sharedInstanceWithDelegate:self];
		[famigoController.view setAutoresizingMask:(UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight)];
		famigoController.view.frame = G_SUBVIEW_FRAME;
	}
	
	[self.view addSubview:famigoController.view];
    [famigoController viewWillAppear:NO];
	[self.famigoController show];
	
	[self hideGearTab];
}

- (void) launchNavigationController {
	if (self.navigationController) {
		self.navigationController = nil;
		[self.navigationController release];
	}
	
	self.navigationController = [[NavigationViewController alloc] initWithNibName:nil bundle:nil];
	[navigationController.view setAutoresizingMask:(UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight)];
	navigationController.view.frame = G_SUBVIEW_FRAME;
	[navigationController viewWillAppear:NO];
	[self.view addSubview:navigationController.view];
	
	[self showGearTab];
}

- (void)famigoReady {
	Famigo *f = [Famigo sharedInstance];
	NSAssert(f.family_id && f.member_id, @"famigoReady, but no FamilyID or MemberID set!!");
	[f loadGame];
}

- (void) showGearTab {
	self.gearTab.enabled = YES;
	self.gearTab.alpha = 1;
	[self.view bringSubviewToFront:self.gearTab];
}

- (void) hideGearTab {
	self.gearTab.enabled = NO;
	self.gearTab.alpha = 0;
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return NO;
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

- (void)dealloc {
    [super dealloc];
}

- (void)famigoNote:(NSNotification*)note {
	NSString *noteName = [note name];
	NSLog(@"received note %@", noteName);
	Famigo *f = [Famigo sharedInstance];
	
	if ([noteName isEqualToString:FamigoMessageGameReady]) {
		// Game Ready means the game is ready to go.  However, we have a bit of setup to do, in terms of
		// selecting a story.  If that hasn't been done yet, ignore this message.
		[self showGearTab];
		[[Quest sharedInstance] rebuildQuest];
		
		[self buildUIForQuest];
	} else if ([noteName isEqualToString:FamigoMessageGameCreated]) {
		// Following game creation, we need to set up our data structures.
		NSDictionary *gameDictionary = [[Quest sharedInstance] buildDictionaryForCurrentQuest];		
		[f.gameInstance setValue:gameDictionary forKey:f.game_name];
		[f updateGame];
		
		[self showGearTab];
	} else if ([noteName isEqualToString:FamigoMessageGameCanceled]) {
		// Wah wah wahhhhh.  Back to Famigo.
		UIAlertView *av = nil;
		av = [[UIAlertView alloc] initWithTitle:@"Uh oh"
										message:@"This round has been canceled.  Time to start another one!" 
									   delegate:self 
							  cancelButtonTitle:@"OK"
							  otherButtonTitles:nil];
		
		[av show];
		[av release];
		
		[self launchFamigoController];
	} else if ([noteName isEqualToString:FamigoMessageGameFinished]) {		
		UIAlertView *av = nil;
		av = [[UIAlertView alloc] initWithTitle:@"Woohoo"
										message:@"You did it!  You're off the island!" 
									   delegate:self 
							  cancelButtonTitle:@"OK"
							  otherButtonTitles:nil];
		
		[av show];
		[av release];
		
		[self launchFamigoController];
	}
}

-(void)buildUIForQuest {
	Quest *q = [Quest sharedInstance];	
	UIAlertView *av = nil;
	
	switch (q.currentStage) {
		case Start:
			// No point in showing this alert view twice.
			if (self.previousStage == Start)
				return;
			
			[self launchShipwreckedLogo];
			
			av = [[UIAlertView alloc] initWithTitle:@"You're shipwrecked!"
														 message:@"Your family has been shipwrecked!  The only way to get off of the island is to work together.  You won't make it unless everyone pitches in!" 
														delegate:self 
											   cancelButtonTitle:@"OK"
											   otherButtonTitles:nil];
			av.tag = StartStageTag;
			[av show];
			[av release];
			break;
		case Organize:
			if (self.previousStage == Organize)
				[self.navigationController refreshView];
			else 
				[self launchNavigationController];

			break;
		default:
			NSAssert(0, @"can't figure out UI for this stage.");
	}
	
	self.previousStage = q.currentStage;
}

-(void)incrementStageOfQuest {
	Quest *q = [Quest sharedInstance];
	q.currentStage++;
	NSDictionary *gameDictionary = [q buildDictionaryForCurrentQuest];		
	Famigo *f = [Famigo sharedInstance];
	[f.gameInstance setValue:gameDictionary forKey:f.game_name];
	[f updateGame];
}

#pragma mark Gear Tab
- (void) launchHelpController {
	[self launchFamigoController];
	[self.famigoController launchHelpController];
}

- (void)cancelGame { 
	UIAlertView *av = [[UIAlertView alloc] initWithTitle:@"Cancel Shipwreck?"
												 message:@"Are you sure? This action can NOT be undone." 
												delegate:self 
									   cancelButtonTitle:@"Cancel Game" 
									   otherButtonTitles:@"Keep Playing",nil];
	av.tag = GearActionTag;
	[av show];
	[av release];
}

- (void)gearAction:(id)sender {
	UIActionSheet *as = nil;
	NSDictionary *gameInstance = [Famigo sharedInstance].gameInstance;
	NSString *gameStatus = [gameInstance valueForKey:@"famigo_active"];
	
	if ([gameStatus isEqualToString:@"finished"] || [gameStatus isEqualToString:@"canceled"]) {
		as = [[UIActionSheet alloc] initWithTitle:nil
										 delegate:self 
								cancelButtonTitle:@"Continue Playing" 
						   destructiveButtonTitle:nil
								otherButtonTitles:@"Back to Famigo",nil];
	}
	else {
		as = [[UIActionSheet alloc] initWithTitle:nil
										 delegate:self
								cancelButtonTitle:@"Continue Playing"
						   destructiveButtonTitle:nil
								otherButtonTitles:@"Back to Famigo",@"Cancel Game",@"Help",nil];
	}
	
	[as showInView:self.view];
	[as release];	
}

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex {
	NSInteger cancelIndex = [actionSheet cancelButtonIndex];
	
	if (buttonIndex != cancelIndex) {
		if (buttonIndex == BackToFamigoButtonIndex) {
			[self launchFamigoController];
		}
		else if (buttonIndex == CancelButtonIndex) {
			[self cancelGame];
		} 
		else if (buttonIndex == HelpButtonIndex) {
			[self launchHelpController];
		}
	}
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	if (alertView.tag == GearActionTag) {
		if (buttonIndex == alertView.cancelButtonIndex) {
			Famigo *f = [Famigo sharedInstance];
			[f cancelGame];
			[self launchFamigoController];
		} else {
			// Nothing to do here.
		}
	}
	if (alertView.tag == StartStageTag) {
		[self.logoView removeFromSuperview];
		[logoView release];
		logoView = nil;
		
		[self incrementStageOfQuest];
	}
}
@end
